class ZillowError(Exception):
    """A ZillowError exception is raised if the API endpoint responded
    with an error code (http://www.zillow.com/howto/api/GetDeepSearchResults.htm).
    """

    code = dict(
        [
            (0, "Request successfully processed"),
            (
                1,
                "Service error-there was a server-side error "
                + "while processing the request. \n "
                + "Check to see if your url is properly formed: delimiters, "
                + "character cases, etc.",
            ),
            (
                2,
                "The specified ZWSID parameter was invalid or "
                + "not specified in the request. \n"
                + "Check if you have provided a ZWSID in your API call. "
                + "If yes, check if the ZWSID is keyed in correctly. "
                + "If it still doesn't work, contact Zillow to "
                + "get help on fixing your ZWSID.",
            ),
            (
                3,
                "Web services are currently unavailable.\n"
                + "The Zillow Web Service is currently not available. "
                + "Please come back later and try again.",
            ),
            (
                4,
                "The API call is currently unavailable.\n"
                + "The Zillow Web Service is currently not available. "
                + "Please come back later and try again.",
            ),
            (6, "This account is not authorized to execute this API call."),
            (7, "Too many requests. \n" + "Daily requests exceeded.",),
            (
                500,
                "Invalid or missing address parameter.\n"
                "Check if the input address parameter "
                + "matches the format specified "
                + "in the input parameters table. When inputting a city name, the "
                + "state should also be given. A city name alone will not give "
                + "a valid address.",
            ),
            (
                501,
                "Invalid or missing citystatezip parameter. "
                + "Check if the input address parameter matches "
                + "the format specified "
                + "in the input parameters table. When inputting a city name, the "
                + "state should also be given. A city name alone will not give"
                + "a valid address.",
            ),
            (
                502,
                "No results found. \n "
                + "Sorry, the address you provided is not found in the Zillow "
                + "property database.",
            ),
            (
                503,
                "Failed to resolve city, state or ZIP code.\n "
                + "Check if the city-state combination is valid. "
                + "Also check if you have provided a valid ZIP code.",
            ),
            (
                504,
                "No coverage for specified area. \nThe specified area "
                + "is not covered by the Zillow property database.",
            ),
            (
                505,
                "Timeout Your request timed out. \nThe server could be "
                + "busy or unavailable. Try again later.",
            ),
            (
                506,
                "Address string too long. \nIf address is valid, try using "
                + "abbreviations.",
            ),
            (
                507,
                "No exact match found. \n"
                + "Verify that the given address is correct.",
            ),
            # this error isn't specified in the zillow doc's,
            # but returned instead of 502
            (508, "No exact match found for input address."),
        ]
    )

    def __init__(self, status, url=None, response=None):
        """Constructor method
        """
        Exception.__init__(self, status)  # Exception is an old-school class
        self.status = status
        self.message = "Status {code}: {text}".format(
            code=status, text=self.code.get(int(status), "Unknown status.")
        )
        self.url = url
        self.response = response

    def __str__(self):
        return self.message


class ZillowFail(Exception):
    """A ZillowFail exception is raised if the API endpoint could not be reached or
       the request did not return valid XML.
    """

    def __init__(self):
        Exception.__init__(self)


class ZillowNoResults(Exception):
    """A ZillowNoResults exception is raised if the request did not return any results.
    """

    def __init__(self):
        Exception.__init__(self)



"""
Tests for `pyzillow` module.
"""

import pytest
import responses

from api_responses import (
    APIReponses,
    set_get_deep_search_response,
    set_updated_property_details_response,
)
from pyzillow.pyzillow import (
    GetDeepSearchResults,
    GetUpdatedPropertyDetails,
    ZillowWrapper,
)
from pyzillow.pyzillowerrors import ZillowError


class TestPyzillow(object):
    @classmethod
    def setup_class(cls):
        cls.ZILLOW_API_KEY = "NO_KEY_NEEDED_DUE_TO_MOCKING_API"
        cls.address = "2114 Bigelow Ave Seattle, WA"
        cls.zipcode = "98109"
        cls.api_response_obj = APIReponses()

    @responses.activate
    def test_zillow_api_connect(self):

        set_get_deep_search_response(
            self.api_response_obj.get("get_deep_search_200_ok")
        )
        zillow_data = ZillowWrapper(self.ZILLOW_API_KEY)
        zillow_search_response = zillow_data.get_deep_search_results(
            self.address, self.zipcode
        )

        assert zillow_search_response.find("message").find("code").text == "0"

    @responses.activate
    def test_zillow_error_invalid_ZWSID(self):
        """
        This test checks the correct error message if no ZWSID is provided.
        Expected error code: 2
        """
        set_get_deep_search_response(self.api_response_obj.get("error_2_zwsid_missing"))
        zillow_data = ZillowWrapper(None)

        with pytest.raises(ZillowError) as excinfo:
            zillow_data.get_deep_search_results(address=self.address, zipcode=None)
        error_msg = "Status 2: The specified ZWSID parameter was invalid"
        assert error_msg in str(excinfo.value)

    @responses.activate
    def test_zillow_error_missing_address(self):
        """
        This test checks the correct error message if no address is provided.
        Expected error code: 500
        """

        set_get_deep_search_response(
            self.api_response_obj.get("error_500_no_address_provided")
        )
        zillow_data = ZillowWrapper(self.ZILLOW_API_KEY)

        with pytest.raises(ZillowError) as excinfo:
            zillow_data.get_deep_search_results(address=self.address, zipcode=None)
        error_msg = "Status 500: Invalid or missing address parameter."
        assert error_msg in str(excinfo.value)

    @responses.activate
    def test_zillow_error_missing_zipcode(self):
        """
        This test checks the correct error message if no zipcode is provided.
        Expected error code: 501
        """

        set_get_deep_search_response(
            self.api_response_obj.get("error_501_no_city_state")
        )
        zillow_data = ZillowWrapper(self.ZILLOW_API_KEY)

        with pytest.raises(ZillowError) as excinfo:
            zillow_data.get_deep_search_results(address=self.address, zipcode=None)
        error_msg = "Status 501: Invalid or missing citystatezip parameter"
        assert error_msg in str(excinfo.value)

    @responses.activate
    def test_zillow_error_no_property_match(self):
        """
        This test checks the correct error message if no property is found.
        Expected error code: 508
        Address and zip code of an exisiting property, but not listed
        """

        address = "not a valid address"
        zipcode = "20001"
        set_get_deep_search_response(
            self.api_response_obj.get("error_508_invalid_address")
        )
        zillow_data = ZillowWrapper(self.ZILLOW_API_KEY)

        with pytest.raises(ZillowError) as excinfo:
            zillow_data.get_deep_search_results(address=address, zipcode=zipcode)
        error_msg = "Status 508: No exact match found for input address."
        assert error_msg in str(excinfo.value)

    # CASE BELOW SEEMS TO HAVE BEEN DEPRECATED
    # # @responses.activate
    # def test_zillow_error_match_zipcode_city(self):
    #     # This test checks the correct error message if no zipcode is provided.
    #     # Expected error code: 503
    #     mismatch_zipcode = '97204'  # Portland, OR
    #     zillow_data = ZillowWrapper(self.ZILLOW_API_KEY)

    #     with pytest.raises(ZillowError) as excinfo:
    #         zillow_data.get_deep_search_results(
    #             address=self.address,
    #             zipcode=mismatch_zipcode
    #         )
    #     error_msg = "Status 508: No exact match found for input address."
    #     assert error_msg in str(excinfo.value)

    @responses.activate
    def test_zillow_error_invalid_zipcode(self):
        """
        This test checks the correct error message if an
        invalid zipcode is provided.
        Expected error code: 508
        """

        invalid_zipcode = "ABCDE"  # invalid zipcode
        set_get_deep_search_response(
            self.api_response_obj.get("error_508_invalid_zipcode")
        )

        zillow_data = ZillowWrapper(self.ZILLOW_API_KEY)

        with pytest.raises(ZillowError) as excinfo:
            zillow_data.get_deep_search_results(
                address=self.address, zipcode=invalid_zipcode
            )
        error_msg = "Status 508: No exact match found for input address."
        assert error_msg in str(excinfo.value)

    @responses.activate
    def test_zillow_error_no_coverage(self):
        """
        This test checks the correct error message
        if no coverage is provided.
        Expected error code: 508
        """

        address = "Calle 21 y O, Vedado, Plaza, Ciudad de la Habana, Cuba"
        zipcode = "10400"  # Cuban address, I assume Zillow doesn't cover Cuba
        set_get_deep_search_response(
            self.api_response_obj.get("error_508_outside_of_area")
        )

        zillow_data = ZillowWrapper(self.ZILLOW_API_KEY)

        with pytest.raises(ZillowError) as excinfo:
            zillow_data.get_deep_search_results(address=address, zipcode=zipcode)
        error_msg = "Status 508: No exact match found for input address."
        assert error_msg in str(excinfo.value)

    @responses.activate
    def test_zillow_error_account_not_authorized(self):
        """
        This test checks for account not authorized error
        Expected error code: 6
        """

        set_get_deep_search_response(
            self.api_response_obj.get("error_6_account_not_authorized")
        )

        zillow_data = ZillowWrapper(self.ZILLOW_API_KEY)

        with pytest.raises(ZillowError) as excinfo:
            zillow_data.get_deep_search_results(
                address=self.address, zipcode=self.zipcode
            )
        error_msg = "Status 6: This account is not authorized to execute this API call"
        assert error_msg in str(excinfo.value)

    @responses.activate
    def test_deep_search_results(self):
        """
        Tests parsing of deep_search results
        """

        address = "2114 Bigelow Ave Seattle, WA"
        zipcode = "98109"

        set_get_deep_search_response(
            self.api_response_obj.get("get_deep_search_200_ok")
        )

        zillow_data = ZillowWrapper(api_key=None)
        deep_search_response = zillow_data.get_deep_search_results(address, zipcode)
        result = GetDeepSearchResults(deep_search_response)

        assert result.zillow_id == "48749425"
        assert result.home_type == "SingleFamily"
        assert (
            result.home_detail_link
            == "http://www.zillow.com/homedetails/"
            + "2114-Bigelow-Ave-N-Seattle-WA-98109/48749425_zpid/"
        )
        assert (
            result.graph_data_link
            == "http://www.zillow.com/homedetails/"
            + "2114-Bigelow-Ave-N-Seattle-WA-98109/"
            + "48749425_zpid/#charts-and-data"
        )
        assert result.map_this_home_link == "http://www.zillow.com/homes/48749425_zpid/"
        assert result.tax_year == "2018"
        assert result.tax_value == "1534000.0"
        assert result.year_built == "1924"
        assert result.property_size == "4680"
        assert result.home_size == "3470"
        assert result.bathrooms == "3.0"
        assert result.bedrooms == "4"
        assert result.last_sold_date == "11/26/2008"
        assert result.last_sold_price_currency == "USD"
        assert result.last_sold_price == "995000"
        lat = float(result.latitude)
        assert lat - 0.01 <= 47.637933 <= lat + 0.01
        lng = float(result.longitude)
        assert lng - 0.01 <= -122.347938 <= lng + 0.01
        assert result.zestimate_amount == "2001121"
        # assert result.zestimate_currency == 'USD'
        assert result.zestimate_last_updated == "05/28/2020"
        assert result.zestimate_value_change == "-16739"
        assert result.zestimate_valuation_range_high == "2121188"
        assert result.zestimate_valuation_range_low == "1881054"
        assert result.zestimate_percentile == "0"

    @responses.activate
    def test_get_updated_property_details_results(self):
        """
        Tests parsing of updated_property_details results
        """

        zillow_id = "48749425"
        set_updated_property_details_response(
            self.api_response_obj.get("updated_property_details_200_ok")
        )

        zillow_data = ZillowWrapper(api_key=None)
        updated_property_details_response = zillow_data.get_updated_property_details(
            zillow_id
        )
        result = GetUpdatedPropertyDetails(updated_property_details_response)

        assert result.zillow_id == "48749425"
        assert result.home_type == "SingleFamily"
        assert (
            result.home_detail_link
            == "http://www.zillow.com/homedetails/"
            + "2114-Bigelow-Ave-N-Seattle-WA-98109/48749425_zpid/"
        )
        assert (
            result.photo_gallery
            == "http://www.zillow.com/homedetails/"
            + "2114-Bigelow-Ave-N-Seattle-WA-98109/"
            + "48749425_zpid/#image=lightbox%3Dtrue"
        )
        assert (
            result.home_info
            == "http://www.zillow.com/homedetails/"
            + "2114-Bigelow-Ave-N-Seattle-WA-98109/48749425_zpid/"
        )
        assert result.year_built == "1924"
        assert result.property_size == "4680"
        assert result.home_size == "3470"
        assert result.bathrooms == "3.0"
        assert result.bedrooms == "4"
        assert result.year_updated == "2003"
        assert result.basement == "Finished"
        assert result.roof == "Composition"
        assert result.view == "Water, City, Mountain"
        assert result.heating_sources == "Gas"
        assert result.heating_system == "Forced air"
        assert (
            result.rooms
            == "Laundry room, Walk-in closet, Master bath, Office,"
            + " Dining room, Family room, Breakfast nook"
        )
        assert result.neighborhood == "Queen Anne"
        assert result.school_district == "Seattle"
        lat = float(result.latitude)
        assert lat - 0.01 <= 47.637933 <= lat + 0.01
        lng = float(result.longitude)
        assert lng - 0.01 <= -122.347938 <= lng + 0.01
        assert result.floor_material is None
        assert result.num_floors == "2"
        assert result.parking_type == "Off-street"
        # assert result.home_description == """Bright, spacious, """

    @classmethod
    def teardown_class(cls):
        pass



#!/usr/bin/env python
#

"""

Distutils setup script for pyzillow.

"""


import os
import sys

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


if sys.argv[-1] == "publish":
    os.system("python setup.py sdist upload")
    sys.exit()

readme = open("README.rst").read()
doclink = """
Documentation
-------------

The full documentation is at https://pyzillow.readthedocs.io/."""
history = open("HISTORY.rst").read().replace(".. :changelog:", "")

setup(
    name="pyzillow",
    version="0.7.0",
    description="Python API wrapper for Zillow's API",
    long_description=readme + "\n\n" + doclink + "\n\n" + history,
    author="Hannes Hapke",
    author_email="hannes.hapke@gmail.com",
    url="https://github.com/hanneshapke/pyzillow",
    packages=["pyzillow"],
    package_dir={"pyzillow": "pyzillow"},
    include_package_data=True,
    install_requires=["requests"],
    license="MIT",
    zip_safe=False,
    keywords=["pyzillow", "zillow", "api", "real estate"],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
)



import requests

from xml.etree import cElementTree as ElementTree  # for zillow API

from .pyzillowerrors import ZillowError, ZillowFail, ZillowNoResults
from . import __version__


class ZillowWrapper(object):
    """This class provides an interface into the Zillow API. An API key is required to create an instance of this class:

    >>> from pyzillow.pyzillow import ZillowWrapper
    >>> zillow_data = ZillowWrapper(YOUR_ZILLOW_API_KEY)

    To request data from Zillow, you can choose between:

        1. The GetDeepSearchResults API endpoint (:class:`pyzillow.pyzillow.GetDeepSearchResults`)
           which requires the following arguments:

            * A street address (e.g. ``'2114 Bigelow Ave'``)
            * A ZIP code or city and state combination (e.g. ``'98109'`` or ``'Seattle, WA'``)
            * Optional: Enabling or disabling Zillow Rentzestimate information in
              API results (``True``/``False``)

            Example:

            >>> from pyzillow.pyzillow import ZillowWrapper, GetDeepSearchResults
            >>> zillow_data = ZillowWrapper(YOUR_ZILLOW_API_KEY)
            >>> deep_search_response = zillow_data.get_deep_search_results(address,
                                                                           zipcode,
                                                                           rentzestimate)
            >>> result = GetDeepSearchResults(deep_search_response)

        2. The GetUpdatedPropertyDetails API endpoint
           (:class:`pyzillow.pyzillow.GetUpdatedPropertyDetails`) which requires a
           Zillow Property ID (ZPID) as an argument. You can acquire this identifier by
           accessing ``.zillow_id`` from a :class:`pyzillow.pyzillow.GetDeepSearchResults`
           object. GetUpdatedPropertyDetails data is not available for all valid Zillow IDs.

            Example:

            >>> from pyzillow.pyzillow import ZillowWrapper, GetUpdatedPropertyDetails
            >>> zillow_data = ZillowWrapper(YOUR_ZILLOW_API_KEY)
            >>> updated_property_details_response = \
                zillow_data.get_updated_property_details(zillow_id)
            >>> result = GetUpdatedPropertyDetails(updated_property_details_response)
    """

    def __init__(self, api_key: str = None):
        """Constructor method
        """
        self.api_key = api_key

    def get_deep_search_results(
        self, address: str, zipcode: str, rentzestimate: bool = False
    ):
        """This method provides results from the GetDeepSearchResults API endpoint as an XML object.

        :param address: Street address to look up
        :type address: str
        :param zipcode: ZIP code to look up
        :type zipcode: str
        :param rentzestimate: Add Rent Zestimate information to result (True/False),
         defaults to False
        :type rentzestimate: bool, optional
        :return: Result from API query
        :rtype: xml.etree.ElementTree.Element
        """
        url = "http://www.zillow.com/webservice/GetDeepSearchResults.htm"

        params = {
            "address": address,
            "citystatezip": zipcode,
            "rentzestimate": str(rentzestimate).lower(),
            "zws-id": self.api_key,
        }
        return self.get_data(url, params)

    def get_updated_property_details(self, zpid: str):
        """This method provides results from the GetUpdatedPropertyDetails API endpoint as an XML object.

        :param zpid: Zillow Web Service Identifier
        :type zpid: str
        :return: Result from API query
        :rtype: xml.etree.ElementTree.Element
        """
        url = "http://www.zillow.com/webservice/GetUpdatedPropertyDetails.htm"

        params = {"zpid": zpid, "zws-id": self.api_key}
        return self.get_data(url, params)

    def get_data(self, url: str, params: dict):
        """This method requests data from the API endpoint specified in the url argument. It uses parameters from the params argument.

        :param url: URL of API endpoint
        :type url: str
        :param params: Parameters for API query
        :type params: dict
        :raises ZillowFail: The API endpoint could not be reached or the request
            did not return valid XML
        :raises ZillowError: The API endpoint responded with an error code
        :raises ZillowNoResults: The request did not return any results
        :return: Result from API query
        :rtype: xml.etree.ElementTree.Element
        """
        try:
            request = requests.get(
                url=url,
                params=params,
                headers={
                    "User-Agent": "".join(["pyzillow/", __version__, " (Python)"])
                },
            )
        except (
            requests.exceptions.ConnectionError,
            requests.exceptions.TooManyRedirects,
            requests.exceptions.Timeout,
        ):
            raise ZillowFail

        try:
            request.raise_for_status()
        except requests.exceptions.HTTPError:
            raise ZillowFail

        try:
            response = ElementTree.fromstring(request.text)
        except ElementTree.ParseError:
            print("Zillow response is not a valid XML ({})".format(params["address"]))
            raise ZillowFail

        if response.findall("message/code")[0].text != "0":
            raise ZillowError(int(response.findall("message/code")[0].text))
        else:
            if not response.findall("response"):
                print("Zillow returned no results for ({})".format(params["address"]))
                raise ZillowNoResults
            return response


class ZillowResults(object):
    """Base class for :class:`pyzillow.pyzillow.GetDeepSearchResults`
       and :class:`pyzillow.pyzillow.GetUpdatedPropertyDetails`.
    """

    attribute_mapping = {}

    def get_attr(self, attr):
        """
        """
        try:
            return self.data.find(self.attribute_mapping[attr]).text
        except AttributeError:
            return None

    def __str__(self):
        return self.zillow_id

    @property
    def area_unit(self):
        """
        lotSizeSqFt
        """
        return u"SqFt"

    @property
    def last_sold_price_currency(self):
        """
        lastSoldPrice currency
        """
        return self.data.find(self.attribute_mapping["last_sold_price"]).attrib[
            "currency"
        ]


class GetDeepSearchResults(ZillowResults):
    """Maps results from the XML data array into attributes of an instance of GetDeepSearchResults.

    An instance of ``GetDeepSearchResults`` has the following attributes:
    ``.bathrooms``
    ``.bedrooms``
    ``.city``
    ``.fips_county``
    ``.graph_data_link``
    ``.home_detail_link``
    ``.home_size``
    ``.home_type``
    ``.last_sold_date``
    ``.last_sold_price``
    ``.latitude``
    ``.longitude``
    ``.map_this_home_link``
    ``.property_size``
    ``.rentzestimate_amount``
    ``.rentzestimate_last_updated``
    ``.rentzestimate_valuation_range_high``
    ``.rentzestimate_valuation_range_low``
    ``.rentzestimate_value_change``
    ``.state``
    ``.street``
    ``.tax_value``
    ``.tax_year``
    ``.total_rooms``
    ``.use_code``
    ``.year_built``
    ``.zestimate_amount``
    ``.zestimate_last_updated``
    ``.zestimate_percentile``
    ``.zestimate_valuation_range_high``
    ``.zestimate_valuation_range_low``
    ``.zestimate_value_change``
    ``.zillow_id``
    ``.zipcode``
    """

    attribute_mapping = {
        "bathrooms": "result/bathrooms",
        "bedrooms": "result/bedrooms",
        "city": "result/address/city",
        "fips_county": "result/FIPScounty",
        "graph_data_link": "result/links/graphsanddata",
        "home_detail_link": "result/links/homedetails",
        "home_size": "result/finishedSqFt",
        "home_type": "result/useCode",
        "last_sold_date": "result/lastSoldDate",
        "last_sold_price": "result/lastSoldPrice",
        "latitude": "result/address/latitude",
        "longitude": "result/address/longitude",
        "map_this_home_link": "result/links/mapthishome",
        "property_size": "result/lotSizeSqFt",
        "rentzestimate_amount": "result/rentzestimate/amount",
        "rentzestimate_last_updated": "result/rentzestimate/last-updated",
        "rentzestimate_valuation_range_high": "result/rentzestimate/valuationRange/high",
        "rentzestimate_valuation_range_low": "result/rentzestimate/valuationRange/low",
        "rentzestimate_value_change": "result/rentzestimate/valueChange",
        "state": "result/address/state",
        "street": "result/address/street",
        "tax_value": "result/taxAssessment",
        "tax_year": "result/taxAssessmentYear",
        "total_rooms": "result/totalRooms",
        "use_code": "result/useCode",
        "year_built": "result/yearBuilt",
        "zestimate_amount": "result/zestimate/amount",
        "zestimate_last_updated": "result/zestimate/last-updated",
        "zestimate_percentile": "result/zestimate/percentile",
        "zestimate_valuation_range_high": "result/zestimate/valuationRange/high",
        "zestimate_valuation_range_low": "result/zestimate/valuationRange/low",
        "zestimate_value_change": "result/zestimate/valueChange",
        "zillow_id": "result/zpid",
        "zipcode": "result/address/zipcode",
    }

    def __init__(self, data, *args, **kwargs):
        """Constructor method
        """
        self.data = data.findall("response/results")[0]
        for attr in self.attribute_mapping.__iter__():
            try:
                self.__setattr__(attr, self.get_attr(attr))
            except AttributeError:
                print("AttributeError with {}".format(attr))

    @property
    def region_name(self):
        """
        region name
        """
        try:
            return self.data.find("result/localRealEstate/region").attrib["name"]
        except AttributeError:
            return None

    @property
    def region_id(self):
        """
        region id
        """
        try:
            return self.data.find("result/localRealEstate/region").attrib["id"]
        except AttributeError:
            return None

    @property
    def region_type(self):
        """
        region type
        """
        try:
            return self.data.find("result/localRealEstate/region").attrib["type"]
        except AttributeError:
            return None


class GetUpdatedPropertyDetails(ZillowResults):
    """Maps results from the XML data array into attributes of an instance of GetUpdatedPropertyDetails.

    An instance of ``GetUpdatedPropertyDetails`` has the following attributes:
    ``.agent_name``
    ``.agent_profile_url``
    ``.appliances``
    ``.basement``
    ``.bathrooms``
    ``.bedrooms``
    ``.brokerage``
    ``.city``
    ``.cooling_system``
    ``.elementary_school``
    ``.exterior_material``
    ``.floor_material``
    ``.heating_sources``
    ``.heating_system``
    ``.high_school``
    ``.home_description``
    ``.home_detail_link``
    ``.home_info``
    ``.home_size``
    ``.home_type``
    ``.latitude``
    ``.longitude``
    ``.middle_school``
    ``.neighborhood``
    ``.num_floors``
    ``.num_rooms``
    ``.page_view_count_this_month``
    ``.page_view_count_total``
    ``.parking_type``
    ``.photo_gallery``
    ``.posting_agent``
    ``.posting_last_update``
    ``.posting_mls``
    ``.posting_status``
    ``.posting_type``
    ``.price``
    ``.property_size``
    ``.roof``
    ``.rooms``
    ``.school_district``
    ``.state``
    ``.street``
    ``.view``
    ``.year_built``
    ``.year_updated``
    ``.zillow_id``
    ``.zipcode``
    """

    attribute_mapping = {
        # attributes in common with GetDeepSearchResults
        "bathrooms": "editedFacts/bathrooms",
        "bedrooms": "editedFacts/bedrooms",
        "city": "result/address/city",
        "home_detail_link": "links/homeDetails",
        "home_size": "editedFacts/finishedSqFt",
        "home_type": "editedFacts/useCode",
        "latitude": "address/latitude",
        "longitude": "address/longitude",
        "property_size": "editedFacts/lotSizeSqFt",
        "state": "result/address/state",
        "street": "result/address/street",
        "year_built": "editedFacts/yearBuilt",
        "zillow_id": "zpid",
        "zipcode": "result/address/zipcode",
        # new attributes in GetUpdatedPropertyDetails
        "agent_name": "posting/agentName",
        "agent_profile_url": "posting/agentProfileUrl",
        "appliances": "editedFacts/appliances",
        "basement": "editedFacts/basement",
        "brokerage": "posting/brokerage",
        "cooling_system": "editedFacts/coolingSystem",
        "elementary_school": "elementarySchool",
        "exterior_material": "editedFacts/exteriorMaterial",
        "floor_material": "editedFacts/floorCovering",
        "heating_sources": "editedFacts/heatingSources",
        "heating_system": "editedFacts/heatingSystem",
        "high_school": "highSchool",
        "home_description": "homeDescription",
        "home_info": "links/homeInfo",
        "middle_school": "middleSchool",
        "neighborhood": "neighborhood",
        "num_floors": "editedFacts/numFloors",
        "num_rooms": "editedFacts/numRooms",
        "page_view_count_this_month": "pageViewCount/currentMonth",
        "page_view_count_total": "pageViewCount/total",
        "parking_type": "editedFacts/parkingType",
        "photo_gallery": "links/photoGallery",
        "photo_gallery": "links/photoGallery",
        "posting_agent": "posting/agentName",
        "posting_last_update": "posting/lastUpdatedDate",
        "posting_mls": "posting/mls",
        "posting_status": "posting/status",
        "posting_type": "posting/type",
        "price": "price",
        "roof": "editedFacts/roof",
        "rooms": "editedFacts/rooms",
        "school_district": "schoolDistrict",
        "view": "editedFacts/view",
        "year_updated": "editedFacts/yearUpdated",
    }

    def __init__(self, data, *args, **kwargs):
        """Constructor method
        """
        self.data = data.findall("response")[0]
        for attr in self.attribute_mapping.__iter__():
            try:
                self.__setattr__(attr, self.get_attr(attr))
            except AttributeError:
                print("AttributeError with {}".format(attr))
